export enum DataType {
  ObjectId = 'ObjectId',
  DateTime = 'DateTime',
  Bool = 'Bool',

  Number = 'Number',
  Integer = 'Integer',
  String = 'String',

  Object = 'Object',
  Array = 'Array',
  Binary = 'Binary'
}

export enum FormatType {
  Currency = 'Currency',
  Percentage = 'Percentage',

  Email = 'Email',
  Url = 'Url',
  Phone = 'Phone',
  Fax = 'Fax',

  IPv4 = 'IPv4',
  IPv6 = 'IPv6',
}

export interface Model {
  name: string;
  attributes: any;
  source?: string;
  model?: any;
  schema?: any;
}

export interface Attribute {
  name: string;
  field: string;
  type: DataType;
  formatType?: FormatType;
  required?: boolean;
  defaultValue?: any;
  primaryKey?: boolean;
  unique?: boolean;
  noinsert?: boolean;
  noupdate?: boolean;
  nopatch?: boolean;
  version?: boolean;
  length?: number;
  precision?: number;
  scale?: number;
  noformat?: boolean;
  ignored?: boolean;
  jsonField?: string;
  link?: string; // sourceNameTwo
  typeOf?: Model;
}

export interface MetaModel {
  model: Model;
  attributeName: string;
  primaryKeys: Attribute[];
  attributes: Attribute[];
  selectableAttributes: Attribute[];
  insertableAttributes: Attribute[];
  updatableAttributes: Attribute[];
  patchableAttributes: Attribute[];
  updatableMap: any;
  patchableMap: any;
  requiredAttributes: Attribute[];
  maxLengthAttributes: Attribute[];
  boolFields: string[];
  dateFields: string[];
  integerFields: string[];
  numberFields: string[];
  percentageFields: string[];
  currencyFields: string[];
  emailFields: string[];
  urlFields: string[];
  phoneFields: string[];
  faxFields: string[];
  ipv4Fields: string[];
  ipv6Fields: string[];
  objectFields: MetaModel[];
  arrayFields: MetaModel[];
  map: any;
  version?: string;
}

export interface MetaModelBuilder {
  build(model: Model): MetaModel;
}
