export interface ValueText {
  value: string;
  text: string;
}
export interface ValueCode {
  value: string;
  code: string;
}
export interface ValueName {
  value: string;
  name: string;
}

export interface DateRange {
  startDate?: Date;
  endDate?: Date;
}
export interface TimeRange {
  startTime?: Date;
  endTime?: Date;
}
export interface NumberRange {
  min?: number;
  max?: number;
  lower?: number;
  upper?: number;
}

export interface SearchModel {
  keyword: string;
  sortField: string;
  sortType: string;
  pageIndex: number;
  pageSize: number;
  initPageSize: number;
  fields: string[];
  excluding: any;
  refId?: string|number;
}
export interface SearchResult<T> {
  itemTotal: number;
  results: T[];
  lastPage: boolean;
}

export interface ErrorMessage {
  field: string;
  code: string;
  message?: string;
}
export enum StatusCode {
  DataNotFound = 0,
  Success = 1,
  Error = 2,
  DuplicateKeyError = 3,
  DataVersionError = 4,
  DataCorruptError = 5,
  ExternalError = 6
}
export interface ResultInfo<T> {
  value: T;
  status: StatusCode;
  message: string;
  errors: ErrorMessage[];
}

export interface DiffModel<T, ID> {
  id: ID;
  oldValue: T;
  newValue: T;
}
export enum Status {
  DataNotFound = 0,
  Success = 1,
  Error = 2,
  DataVersionError = 4,
}
